
using System;



public class Parser {
	public const int _EOF = 0;
	public const int _number = 1;
	public const int maxT = 8;

	const bool T = true;
	const bool x = false;
	const int minErrDist = 2;
	
	public Scanner scanner;
	public Errors  errors;

	public Token t;    // last recognized token
	public Token la;   // lookahead token
	int errDist = minErrDist;

public int result = 0;


	public Parser(Scanner scanner) {
		this.scanner = scanner;
		errors = new Errors();
	}

	void SynErr (int n) {
		if (errDist >= minErrDist) errors.SynErr(la.line, la.col, n);
		errDist = 0;
	}

	public void SemErr (string msg) {
		if (errDist >= minErrDist) errors.SemErr(t.line, t.col, msg);
		errDist = 0;
	}
	
	void Get () {
		for (;;) {
			t = la;
			la = scanner.Scan();
			if (la.kind <= maxT) { ++errDist; break; }

			la = t;
		}
	}
	
	void Expect (int n) {
		if (la.kind==n) Get(); else { SynErr(n); }
	}
	
	bool StartOf (int s) {
		return set[s, la.kind];
	}
	
	void ExpectWeak (int n, int follow) {
		if (la.kind == n) Get();
		else {
			SynErr(n);
			while (!StartOf(follow)) Get();
		}
	}


	bool WeakSeparator(int n, int syFol, int repFol) {
		int kind = la.kind;
		if (kind == n) {Get(); return true;}
		else if (StartOf(repFol)) {return false;}
		else {
			SynErr(n);
			while (!(set[syFol, kind] || set[repFol, kind] || set[0, kind])) {
				Get();
				kind = la.kind;
			}
			return StartOf(syFol);
		}
	}

	
	void Expr() {
		Plus(ref result);
	}

	void Plus(ref int a) {
		int b = 0; 
		Prod(ref a);
		while (la.kind == 2 || la.kind == 3) {
			if (la.kind == 2) {
				Get();
				Prod(ref b);
				a += b; 
			} else {
				Get();
				Prod(ref b);
				a -= b; 
			}
		}
	}

	void Prod(ref int a) {
		int b = 0; 
		Fact(ref a);
		while (la.kind == 4 || la.kind == 5) {
			if (la.kind == 4) {
				Get();
				Fact(ref b);
				a *= b; 
			} else {
				Get();
				Fact(ref b);
				a = (int)((double)a / (double)b); 
			}
		}
	}

	void Fact(ref int a) {
		if (la.kind == 1) {
			Number(ref a);
		} else if (la.kind == 6) {
			Get();
			Plus(ref a);
			Expect(7);
		} else SynErr(9);
	}

	void Number(ref int a) {
		Expect(1);
		a = int.Parse(t.val); 
	}



	public void Parse() {
		la = new Token();
		la.val = "";		
		Get();
		Expr();

    Expect(0);
	}
	
	static readonly bool[,] set = {
		{T,x,x,x, x,x,x,x, x,x}

	};
} // end Parser


public class Errors {
	public int count = 0;                                    // number of errors detected
	public System.IO.TextWriter errorStream = Console.Out;   // error messages go to this stream
  public string errMsgFormat = "-- line {0} col {1}: {2}"; // 0=line, 1=column, 2=text
  
	public void SynErr (int line, int col, int n) {
		string s;
		switch (n) {
			case 0: s = "EOF expected"; break;
			case 1: s = "number expected"; break;
			case 2: s = "\"+\" expected"; break;
			case 3: s = "\"-\" expected"; break;
			case 4: s = "\"*\" expected"; break;
			case 5: s = "\"/\" expected"; break;
			case 6: s = "\"(\" expected"; break;
			case 7: s = "\")\" expected"; break;
			case 8: s = "??? expected"; break;
			case 9: s = "invalid Fact"; break;

			default: s = "error " + n; break;
		}
		errorStream.WriteLine(errMsgFormat, line, col, s);
		count++;
	}

	public void SemErr (int line, int col, string s) {
		errorStream.WriteLine(errMsgFormat, line, col, s);
		count++;
	}
	
	public void SemErr (string s) {
		errorStream.WriteLine(s);
		count++;
	}
	
	public void Warning (int line, int col, string s) {
		errorStream.WriteLine(errMsgFormat, line, col, s);
	}
	
	public void Warning(string s) {
		errorStream.WriteLine(s);
	}
} // Errors


public class FatalError: Exception {
	public FatalError(string m): base(m) {}
}

